﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Configuration;
using Microsoft.Win32;
using System.IO.Ports;
using System.Security.Principal;

namespace KPL
{
    public partial class PortSettings : Form
    {
        public PortSettings()
        {
            InitializeComponent();
            InitSettings();
        }

        private void InitSettings()
        {
            try
            {
                foreach (COMPortInfo comPort in COMPortInfo.GetCOMPortsInfo())
                {
                    cmbPorts.Items.Add(comPort.Name);  
                }

                bool rememberSettings = false;
                bool.TryParse(ConfigurationManager.AppSettings["RememberSettings"].ToString(), out rememberSettings);

                bool useHttps = false;
                bool.TryParse(ConfigurationManager.AppSettings["UseHttps"].ToString(), out useHttps);

                if (rememberSettings)
                {
                    cmbPorts.SelectedIndex = cmbPorts.FindString(ConfigurationManager.AppSettings["Port"].ToString());
                    cmbBaudRate.SelectedIndex = cmbBaudRate.FindString(ConfigurationManager.AppSettings["BaudRate"].ToString());
                    cmbDataBits.SelectedIndex = cmbDataBits.FindString(ConfigurationManager.AppSettings["DataBits"].ToString());
                    cmbParity.SelectedIndex = cmbParity.FindString(ConfigurationManager.AppSettings["Parity"].ToString());
                    cmbFlowControl.SelectedIndex = cmbFlowControl.FindString(ConfigurationManager.AppSettings["FlowControl"].ToString());
                    cmbStopBit.SelectedIndex = cmbStopBit.FindString(ConfigurationManager.AppSettings["StopBit"].ToString());
                    txtServicePort.Text = ConfigurationManager.AppSettings["WCFPort"].ToString();
                    chkUseHttps.Checked = useHttps;
                }
                chkRememberSettings.Checked = rememberSettings;
                var identity = WindowsIdentity.GetCurrent();
                var principal = new WindowsPrincipal(identity);
                chkUseHttps.Enabled = principal.IsInRole(WindowsBuiltInRole.Administrator);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);   
            }
        }

        private void SaveSettings()
        {
            try
            {
                System.Configuration.Configuration config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
                config.AppSettings.Settings["Port"].Value = cmbPorts.Text;
                config.AppSettings.Settings["BaudRate"].Value = cmbBaudRate.Text;
                config.AppSettings.Settings["DataBits"].Value = cmbDataBits.Text;
                config.AppSettings.Settings["Parity"].Value = cmbParity.Text;
                config.AppSettings.Settings["FlowControl"].Value = cmbFlowControl.Text;
                config.AppSettings.Settings["StopBit"].Value = cmbStopBit.Text;
                config.AppSettings.Settings["RememberSettings"].Value = chkRememberSettings.Checked.ToString();
                config.AppSettings.Settings["WCFPort"].Value = txtServicePort.Text.Trim();
                config.AppSettings.Settings["UseHttps"].Value = chkUseHttps.Checked.ToString();
                config.Save(ConfigurationSaveMode.Modified);
                ConfigurationManager.RefreshSection("appSettings");
            }
            catch (Exception ex)
            {
                //MessageBox.Show(ex.Message);
            }
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            int servicePort = -1;
            if (!Int32.TryParse(txtServicePort.Text, out servicePort))
            {
                MessageBox.Show("Invalid Service Port!"); 
                return;
            }
            SaveSettings();
            //Only save the settings
            this.DialogResult = System.Windows.Forms.DialogResult.No; 
        }

        private void btnOpenPort_Click(object sender, EventArgs e)
        {
            string port = cmbPorts.Text;
            if (!string.IsNullOrEmpty(port))
            {
                Parity parity = Parity.Space;
                switch (cmbParity.Text)
                {
                    case "None":
                        parity = Parity.None;
                        break;
                    case "Odd":
                        parity = Parity.Odd;
                        break;
                    case "Even":
                        parity = Parity.Even;
                        break;
                    case "Mark":
                        parity = Parity.Mark;
                        break;
                }

                StopBits stopBits = StopBits.None;

                switch (cmbStopBit.Text)
                {
                    case "1":
                        stopBits = StopBits.One;
                        break;
                    case "1.5":
                        stopBits = StopBits.OnePointFive;
                        break;
                    case "2":
                        stopBits = StopBits.Two;
                        break;
                }

                Handshake handshake = Handshake.None;

                switch (cmbFlowControl.Text)
                {
                    case "CTS/RTS":
                        handshake = Handshake.RequestToSend;
                        break;
                    case "XOnXOff":
                        handshake = Handshake.XOnXOff;
                        break;
                }

                try
                {
                    SerialPort serPort = new SerialPort();
                    serPort.PortName = port;
                    serPort.BaudRate = int.Parse(cmbBaudRate.Text);
                    serPort.DataBits = int.Parse(cmbDataBits.Text); 
                    serPort.StopBits = stopBits;
                    serPort.Parity = parity;
                    serPort.Handshake = handshake;
                    serPort.Open();
                    if (serPort.IsOpen)
                    {
                        serPort.Close();  
                    }

                    SaveSettings();
                    //Save the settings and apply them to the current serial port
                    this.DialogResult = System.Windows.Forms.DialogResult.OK;
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Invalid port settings! " + ex.Message);
                }
            }
        }
    }
}
