﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Configuration;
using System.ServiceModel;
using System.ServiceModel.Description;
using KPL.WCF;
using System.ServiceModel.Web;
using CyUSB;
using System.Threading;
using System.Drawing.Imaging;
using System.IO.Ports;
using Microsoft.Win32;
using FTD2XX_NET;
using TokenScanner;
using NLcpfw_wrapper;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.Security.Cryptography.X509Certificates;
using MAGDLL; //new mag class
using PDFDLL; //new pdf417 class
using System.Security.Principal;
using System.Net;
using System.Net.Security;
using System.Management;
using System.ComponentModel;

namespace KPL
{
    public partial class TestForm : Form, IContent
    {
        #region Variables

        private static string tempFolderPath = Path.GetTempPath();

        private string _imageName;
        WebServiceHost _host = null;
        private delegate void UpdateUICallback();
        private delegate void ExceptionCallback();
        private delegate void Prepare_STCAP();
        private delegate void Image_View();
        private USBDeviceList usbDevices;
        private CyUSBDevice MyDevice;
        private CyBulkEndPoint inEndpoint = null;
        private CyControlEndPoint ep0 = null;
        private int BufNum = 0;
        private int BufSz;
        private int QueueSz;
        private int Successes;
        private int Failures;
        private int TotalPixSize;
        private double XferBytes;
        private bool bRunning;
        private bool bInitM280;
        private bool bSysBusy;
        private byte[][] DataBuf;
        private byte[] M280Status = new byte[2];
        private int DatumPos_sX = 0;
        private int DatumPos_sY = 0;
        private int CropSize_X = 0;
        private int CropSize_Y = 0;
        private int AngleInfo = 0;
        private Bitmap CroppedImage;
        private Thread tStartCap;
        private UpdateUICallback updateUI;
        private ExceptionCallback handleException;
        private Prepare_STCAP Pre_STCAP;
        private Image_View Img_View;
        bool ScanerBusy = false;
        public delegate void serialPortFunc(string recvStr);
        private string _previousContent = string.Empty;
        private string _currentContent = string.Empty;
        private string _currentImageContent = string.Empty;
        private string _currentFaceImageContent = string.Empty;
        private string _currentSignatureImageContent = string.Empty;
        private string _isImage = "0";
        Parser _p;
        string _hexValue = string.Empty;
        byte[] MagBuff = new byte[315]; //Magstrip reader buffer
        byte[] buff = new byte[2000]; // PDF417 barcode reader buffer 
        byte[] Rawbuff = new byte[2000]; // RawData buffer 

        string displayString;
        //this is settings for Eseek
        //int ss1 = 26; //start sentinel 1
        //int ss2 = 27; //start sentinel 2
        //int ss3 = 28; //start sentinel 3
        //int MBD = 29; //Magstripe Barcode Designator
        //this is settings for UFCOM M430
        int ss1 = 0; //start sentinel 1
        int ss2 = 1; //start sentinel 2
        int ss3 = 2; //start sentinel 3
        int MBD = 3; //Magstripe Barcode Designator
        byte[] UFCOMbuff = new byte[1000]; //reads data from COM port scanner during initialization

        string rawdata; //holds hex encoded raw data 
        string ASCIIdata; // holds eyeball encoded data
        DateTime d1 = DateTime.Now;
        Dictionary<string, string> fields;

        private static int WM_QUERYENDSESSION = 0x11;

        /*Declaring Scanner Objects*/
        private NetScanW.SLibEx _sLib;
        private NetScanWex.SLibEx _sLibEx;

        /*Declaring Image Objects*/
        private NetScanW.CImage _image;
        private NetScanWex.CImage _imageEx;

        /*Declaring ID Objects*/
        private NetScanW.IdData _idData;
        private NetScanWex.IdData _idDataEx;

        private string filePath = string.Empty;
        private string filePath_back = string.Empty;
        private bool _isEseek = false;

        private NetScanW.CBarCode _barCode;
        private NetScanWex.CBarCode _barCodeEx;

        private bool isBarcodeRead = false;
        private bool isOCRRead = false;

        private string filePath_front_Reformatted = string.Empty;
        private string filePath_back_Reformatted = string.Empty;

        private bool _useAcuantBarcodeParser = true;
        string _faceImage = tempFolderPath + "faceImage.jpg";
        string _signImage = tempFolderPath + "signImage.jpg";

        private NetScanW.CPassport _passport;
        private NetScanWex.CPassport _passportEx;

        private bool _loaded = false;

        public static bool my420; //controls M420 and H325 scanner communications
        public static bool my406; //controls F406 scanner comm.

        private bool scannerInit = false;

        public const ushort USB_PID = 0x0280;
        public const ushort USB3_PID = 0X0282;
        public const ushort USB_VID = 0x028A6;

        #endregion

        #region M420 Variables
        private IntPtr m_hPort = IntPtr.Zero;
        private CPFW.NTF_DATA_delegate NTF_DATA_event = null;
        string msg = string.Empty;
        byte[] Scanbuff = new byte[1000]; //TokenScanner has a 1500 byte internal array.
        int Lng;
        bool OK = false; //control access to scanner and library
        int ParseRet = 0; // return code of parser, determines license status

        #endregion

        static bool bRelease = true;

        public TestForm()
        {
        //    FtdiReload();

            InitializeComponent();

            try
            {
                this.WindowState = FormWindowState.Minimized;
                _isEseek = ConfigurationManager.AppSettings["ScannerType"].ToString().ToLower().Equals("tokenworks");
                _useAcuantBarcodeParser = ConfigurationManager.AppSettings["ParsingMode"].ToString().ToLower().Equals("acuant");
                eSeekToolStripMenuItem.Checked = _isEseek;

                acuantToolStripMenuItem.Checked = !_isEseek;

                this.Text = "Eyemetric Tray     Mode: " + ConfigurationManager.AppSettings["ScannerType"].ToString();

                if (!_isEseek)
                {
                    string scanner = CSSNConstants.GetScannerNameByType(_sLib.ScannerType);
                    if (!string.IsNullOrEmpty(scanner))
                    {
                        this.Text += "    Type: " + scanner;
                    }
                }

                AppContext.Instance().Content = this;
                HandleServiceHost();
                HandleScanner();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                _loaded = true;
            }
        }

        private void HandleScanner()
        {
            try
            {
                if (!_isEseek)
                {
                    statusStrip1.Visible = false;

                    // Scanner objects
                    _sLib = new NetScanW.SLibEx();
                    _sLibEx = new NetScanWex.SLibEx();

                    // Image Object

                    _image = new NetScanW.CImage();
                    _imageEx = new NetScanWex.CImage();

                    // Data objects
                    _idData = new NetScanW.IdData();
                    _idDataEx = new NetScanWex.IdData();

                    //Bar Code Class
                    _barCode = new NetScanW.CBarCode();
                    _barCodeEx = new NetScanWex.CBarCode();

                    _passport = new NetScanW.CPassport();
                    _passportEx = new NetScanWex.CPassport();

                    _InitSDK();

                    if (!System.IO.Directory.Exists(tempFolderPath))
                    {
                        System.IO.Directory.CreateDirectory(tempFolderPath);
                    }
                    filePath = tempFolderPath + "scannedImage.png";
                    filePath_back = tempFolderPath + "scannedImage-back.png";

                    filePath_front_Reformatted = tempFolderPath + "scannedImage_Reformatted.png";
                    filePath_back_Reformatted = tempFolderPath + "scannedImage-back_Reformatted.png";
                }
                else
                {
                    ReaderBuffMgr.GetTimeStamp();
                    statusStrip1.Visible = true;

                    this.usbDevices = new USBDeviceList(1);

                    bool errorOccurred = false;

                    string port = ConfigurationManager.AppSettings["Port"].ToString();
                    if (!string.IsNullOrEmpty(port))
                    {
                        try
                        {
                            OpenPort();
                        }
                        catch (Exception ex)
                        {
                            MessageBox.Show(ex.Message);
                            errorOccurred = true;
                        }
                    }

                    InitM260();


                    m_hPort = TokenScanner.TkwkScan.OpenReader();
                    if (m_hPort != IntPtr.Zero)
                    {
                        Init420();

                        if (errorOccurred)
                        {
                            ConfigureScanner();
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void Execute(string cmd)
        {
            try
            {
                Process p = new Process();
                p.StartInfo.CreateNoWindow = true;
                p.StartInfo.Verb = "runas";

                p.StartInfo.Arguments = cmd;
                p.StartInfo.FileName = "netsh.exe";
                p.StartInfo.UseShellExecute = false;
                p.StartInfo.RedirectStandardOutput = true;
                p.Start();
                p.WaitForExit();
            }
            catch
            {
            }
        }

        private void HandleServiceHost()
        {

            try
            {
                if ((_host != null) && (_host.State == CommunicationState.Opened))
                {
                    _host.Close();
                }

                bool useHttps = false;
                bool.TryParse(ConfigurationManager.AppSettings["UseHttps"].ToString(), out useHttps);

                Uri u;

                if (!useHttps)
                {
                    u = new Uri("http://localhost:" + ConfigurationManager.AppSettings["WCFPort"].ToString());
                }
                else
                {
                    u = new Uri("https://localhost:" + ConfigurationManager.AppSettings["WCFPort"].ToString());
                }

                int osMajorVersion = Environment.OSVersion.Version.Major;
                string cmd = string.Empty;

                _host = new WebServiceHost(typeof(Service), u);
                WebHttpBinding binder = new WebHttpBinding();
                binder.CrossDomainScriptAccessEnabled = true;

                var identity = WindowsIdentity.GetCurrent();
                var principal = new WindowsPrincipal(identity);

                if (principal.IsInRole(WindowsBuiltInRole.Administrator))
                {
                    Execute("http delete urlacl url=http://+:" + ConfigurationManager.AppSettings["WCFPort"].ToString() + "/");
                    Execute("http delete urlacl url=https://+:" + ConfigurationManager.AppSettings["WCFPort"].ToString() + "/");
                }

                if (useHttps)
                {
                    binder.Security.Mode = WebHttpSecurityMode.Transport;

                    ServiceMetadataBehavior smb = new ServiceMetadataBehavior();
                    smb.HttpsGetEnabled = true;
                    smb.HttpsGetUrl = u;
                    _host.Description.Behaviors.Add(smb);

                    if (principal.IsInRole(WindowsBuiltInRole.Administrator))
                    {
                        _host.Credentials.ServiceCertificate.Certificate = new X509Certificate2("Eyemetric.p12", "", X509KeyStorageFlags.MachineKeySet | X509KeyStorageFlags.PersistKeySet | X509KeyStorageFlags.PersistKeySet | X509KeyStorageFlags.Exportable);

                        //add certificate binding to the port

                        var store = new X509Store(StoreName.My, StoreLocation.LocalMachine);
                        store.Open(OpenFlags.ReadWrite);
                        store.Remove(_host.Credentials.ServiceCertificate.Certificate);
                        store.Add(_host.Credentials.ServiceCertificate.Certificate);
                        store.Close();

                        store = new X509Store(StoreName.Root, StoreLocation.LocalMachine);
                        store.Open(OpenFlags.ReadWrite);
                        store.Remove(_host.Credentials.ServiceCertificate.Certificate);
                        store.Add(_host.Credentials.ServiceCertificate.Certificate);
                        store.Close();

                        cmd = "http delete sslcert ipport=0.0.0.0:[port]";
                        cmd = cmd.Replace("[port]", ConfigurationManager.AppSettings["WCFPort"].ToString());
                        Execute(cmd);

                        cmd = "http add sslcert ipport=0.0.0.0:[port] certhash=[hash] appid={89112233-4455-6677-8899-AABBCCDDEEFF}";
                        cmd = cmd.Replace("[port]", ConfigurationManager.AppSettings["WCFPort"].ToString()).Replace("[hash]", _host.Credentials.ServiceCertificate.Certificate.Thumbprint.Trim());
                        Execute(cmd);

                        cmd = "http add urlacl url=https://+:" + ConfigurationManager.AppSettings["WCFPort"].ToString() + "/" + " user=BUILTIN\\Users";
                        Execute(cmd);
                    }
                }
                else
                {
                    if (principal.IsInRole(WindowsBuiltInRole.Administrator))
                    {
                        cmd = "http add urlacl url=http://+:" + ConfigurationManager.AppSettings["WCFPort"].ToString() + "/" + " user=BUILTIN\\Users";
                        Execute(cmd);
                    }
                }

                ServiceEndpoint ep = _host.AddServiceEndpoint(typeof(IService), binder, string.Empty);

                ServiceDebugBehavior stp = _host.Description.Behaviors.Find<ServiceDebugBehavior>();
                stp.HttpHelpPageEnabled = false;
                _host.Open();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }


        private void InitM260()
        {
            try
            {
                this.updateUI = new UpdateUICallback(this.StatusUpdate);
                this.handleException = new ExceptionCallback(this.ThreadException);
                this.Img_View = new Image_View(this.ImageView);
                this.usbDevices.DeviceAttached += new EventHandler(this.usbDevices_DeviceAttached);
                this.usbDevices.DeviceRemoved += new EventHandler(this.usbDevices_DeviceRemoved);

                this.SetDevice();
                this.TotalPixSize = 1572864;
                if (this.TotalPixSize % 12288 != 0)
                {
                    MessageBox.Show("Please check pixel size!");
                }
                this.BufNum = this.TotalPixSize / 12288;
                this.timCheckStatus.Tick += new EventHandler(this.CheckState_Tick);
                this.timCheckStatus.Interval = 100;
                this.bInitM280 = false;
                this.bSysBusy = false;

                this.lblApplicationVersion.Text = "2.0.0.95";
                this.lblApplicationVersion.Refresh();

                if (this.MyDevice != null && this.ep0 != null)
                {
                    int num = 10;
                    byte[] bytes = new byte[num];
                    this.ReadCommand(235, ref bytes, ref num, 0, 1);
                    this.lblFirmwareVersion.Text = Encoding.Default.GetString(bytes);
                    this.lblFirmwareVersion.Refresh();
                }
            }
            catch (Exception)
            {
            }
        }

        private void usbDevices_DeviceRemoved(object sender, EventArgs e)
        {
            try
            {
                this.MyDevice = null;
                this.inEndpoint = null;
                this.SetDevice();
            }
            catch (Exception ex)
            {
                MessageBox.Show("DeviceRemoved : " + ex.Message);
            }
        }

        private void usbDevices_DeviceAttached(object sender, EventArgs e)
        {
            try
            {
                BackgroundWorker bg = new BackgroundWorker();
                bg.DoWork += Bg_DoWork;
                bg.RunWorkerAsync();

                this.SetDevice();
            }
            catch (Exception ex)
            {
                MessageBox.Show("DeviceAttached : " + ex.Message);
            }
        }

        private void Bg_DoWork(object sender, DoWorkEventArgs e)
        {
            Thread.Sleep(1000);
            try
            {
                OpenPort();
            }
            catch (Exception ex)
            {
            }
        }

        private void SetDevice()
        {
            this.MyDevice = (this.usbDevices[USB_VID, USB_PID] as CyUSBDevice);
            if (this.MyDevice == null)
            {
                this.MyDevice = (this.usbDevices[USB_VID, USB3_PID] as CyUSBDevice);
            }

            if (this.MyDevice != null)
            {
                if (this.inEndpoint == null)
                {
                    labUSBState.Text = "Connected";
                    this.labUSBState.BackColor = Color.YellowGreen;
                    this.inEndpoint = (this.MyDevice.EndPointOf(134) as CyBulkEndPoint);
                    this.ep0 = this.MyDevice.ControlEndPt;
                    this.inEndpoint.TimeOut = 1500u;
                    this.ep0.TimeOut = 1500u;
                    this.GetImageInfo();
                    this.SendCommand(225, 0, 0);
                    Thread.Sleep(1000);
                    this.timCheckStatus.Start();
                }
            }
            else
            {
                labUSBState.Text = "Disconnected";
                this.labUSBState.BackColor = Color.Red;
                this.inEndpoint = null;
                this.ep0 = null;
            }
        }

        private void CheckState_Tick(object sender, EventArgs e)
        {
            try
            {
                this.timCheckStatus.Stop();
                int num = 2;
                if (this.MyDevice != null && this.ep0 != null && this.inEndpoint != null)
                {
                    this.ReadCommand(234, ref this.M280Status, ref num, 0, 0);
                    ushort num2 = BitConverter.ToUInt16(this.M280Status, 0);
                    if ((num2 & 8) == 8)
                    {
                        this.btnCapture_Click(0, null);
                    }
                    if ((num2 & 2) == 2)
                    {
                    }
                    if ((num2 & 1) == 1)
                    {
                    }
                    if ((num2 & 16) == 16)
                    {
                        this.bInitM280 = true;
                    }
                    else
                    {
                        this.bInitM280 = false;
                    }
                    if ((num2 & 32) == 32)
                    {
                        this.bSysBusy = true;
                    }
                    else
                    {
                        this.bSysBusy = false;
                    }
                }

                this.timCheckStatus.Start();
            }
            catch (Exception ex)
            {
                MessageBox.Show("CheckState_Tick : " + ex.Message);
            }
        }

        public void GetImageInfo()
        {
            if (this.MyDevice != null && this.ep0 != null && this.inEndpoint != null)
            {
                int num = 20;
                byte[] array = new byte[num];
                if (this.ReadCommand(238, ref array, ref num, 0, 0))
                {
                    Array.Reverse(array, 0, 2);
                    Array.Reverse(array, 2, 2);
                    Array.Reverse(array, 4, 2);
                    Array.Reverse(array, 6, 2);
                    Array.Reverse(array, 14, 2);
                    this.DatumPos_sX = (int)BitConverter.ToUInt16(array, 0);
                    this.DatumPos_sY = (int)BitConverter.ToUInt16(array, 2);
                    this.CropSize_X = (int)BitConverter.ToUInt16(array, 4);
                    this.CropSize_Y = (int)BitConverter.ToUInt16(array, 6);
                    this.AngleInfo = (int)array[13];
                }
                else
                {
                    //MessageBox.Show("Can not find Image Info Please check Firmware version!");
                    this.DatumPos_sX = 66;
                    this.DatumPos_sY = 102;
                    this.CropSize_X = 892;
                    this.CropSize_Y = 564;
                    this.AngleInfo = 127;
                }
            }
        }

        public bool SendCommand(byte Cmd, ushort val, ushort index)
        {
            bool flag = false;
            bool result;
            if (this.MyDevice == null || this.ep0 == null)
            {
                result = flag;
            }
            else
            {
                this.ep0.Target = 0;
                this.ep0.ReqType = 64;
                this.ep0.Direction = 0;
                this.ep0.ReqCode = Cmd;
                this.ep0.Value = val;
                this.ep0.Index = index;
                int num = 0;
                byte[] array = new byte[1];
                flag = this.ep0.XferData(ref array, ref num);
                result = flag;
            }
            return result;
        }

        public bool ReadCommand(byte Cmd, ref byte[] Read, ref int Len, ushort val, ushort index)
        {
            bool flag = false;
            bool result;
            if (this.MyDevice == null || this.ep0 == null)
            {
                result = flag;
            }
            else
            {
                this.ep0.Target = 0;
                this.ep0.ReqType = 64;
                this.ep0.Value = val;
                this.ep0.Index = index;
                this.ep0.ReqCode = Cmd;
                flag = this.ep0.Read(ref Read, ref Len);
                result = flag;
            }
            return result;
        }

        public void XferThread()
        {
            byte[][] cBufs = new byte[this.QueueSz][];
            byte[][] xBufs = new byte[this.QueueSz][];
            byte[][] oLaps = new byte[this.QueueSz][];
            ISO_PKT_INFO[][] pktsInfo = new ISO_PKT_INFO[this.QueueSz][];
            int num = 0;
            try
            {
                this.LockNLoad(ref num, cBufs, xBufs, oLaps, pktsInfo);
            }
            catch (Exception ex)
            {
                ex.GetBaseException();
                if (!base.IsHandleCreated)
                {
                    base.CreateControl();
                }
                base.Invoke(this.handleException);
            }
        }

        public void CheckStatusThread()
        {
            try
            {
                this.CheckStatus();
            }
            catch (Exception ex)
            {
                ex.GetBaseException();
                if (!base.IsHandleCreated)
                {
                    base.CreateControl();
                }
                base.Invoke(this.handleException);

            }
        }

        public unsafe void LockNLoad(ref int j, byte[][] cBufs, byte[][] xBufs, byte[][] oLaps, ISO_PKT_INFO[][] pktsInfo)
        {
            cBufs[j] = new byte[38];
            xBufs[j] = new byte[this.BufSz];
            oLaps[j] = new byte[20];
            pktsInfo[j] = new ISO_PKT_INFO[24];
            fixed (byte* ptr = oLaps[j], ptr2 = cBufs[j], ptr3 = xBufs[j])
            {
                OVERLAPPED* ptr4 = (OVERLAPPED*)ptr;
                ptr4->hEvent = PInvoke.CreateEvent(0u, 0u, 0u, 0u);
                int bufSz = this.BufSz;
                this.inEndpoint.BeginDataXfer(ref cBufs[j], ref xBufs[j], ref bufSz, ref oLaps[j]);
                j++;
                if (j < this.QueueSz)
                {
                    this.LockNLoad(ref j, cBufs, xBufs, oLaps, pktsInfo);
                }
                else
                {
                    this.XferData(cBufs, xBufs, oLaps, pktsInfo);
                }
            }
        }

        public void CheckStatus()
        {
            int num = 2;
            while (true)
            {
                Thread.Sleep(50);
                if (this.MyDevice == null || this.ep0 == null)
                {
                    break;
                }
                this.ep0.Target = 0;
                this.ep0.ReqType = 64;
                this.ep0.Value = 0;
                this.ep0.Index = 0;
                this.ep0.ReqCode = 234;
                this.ep0.Read(ref this.M280Status, ref num);
                ushort num2 = BitConverter.ToUInt16(this.M280Status, 0);
                if ((num2 & 8) == 8)
                {
                    base.Invoke(this.Pre_STCAP);
                }
                if ((num2 & 16) == 16)
                {
                    Thread.Sleep(50);
                }
            }
        }

        public unsafe void XferData(byte[][] cBufs, byte[][] xBufs, byte[][] oLaps, ISO_PKT_INFO[][] pktsInfo)
        {
            try
            {
                int num = 0;
                int num2 = 0;
                int num3 = 0;
                this.Successes = 0;
                this.Failures = 0;
                this.XferBytes = 0.0;

                while (this.bRunning)
                {
                    unsafe
                    {
                        fixed (byte* ptr = oLaps[num])
                        {
                            OVERLAPPED* ptr2 = (OVERLAPPED*)ptr;
                            if (!this.inEndpoint.WaitForXfer(ptr2->hEvent, 1000u))
                            {
                                this.inEndpoint.Abort();
                                PInvoke.WaitForSingleObject(ptr2->hEvent, 500u);
                            }
                        }
                    }

                    if (this.inEndpoint.FinishDataXfer(ref cBufs[num], ref xBufs[num], ref num2, ref oLaps[num]))
                    {
                        this.XferBytes += (double)num2;
                        this.Successes++;
                        Array.Copy(xBufs[num], 0, this.DataBuf[num3], 0, num2);
                        num3++;
                    }
                    else
                    {
                        this.Failures++;
                    }

                    int len = this.BufSz;
                    this.inEndpoint.BeginDataXfer(ref cBufs[num], ref xBufs[num], ref len, ref oLaps[num]);

                    num++;
                    if (num == this.QueueSz)
                    {
                        num = 0;
                        Thread.Sleep(1);
                        if (this.Failures == 0)
                        {
                            base.Invoke(this.Img_View);
                        }
                        else if (Successes == 0)
                        {
                            // System not ready just ignore
                            // Vacate the trash.
                            len = 1024 * 4;
                            byte[] buf = new byte[len];
                            inEndpoint.Abort();
                            inEndpoint.Reset();
                            inEndpoint.XferData(ref buf, ref len);
                            Thread.Sleep(100);
                        }
                        else
                        {
                            // Scanner busy

                            // Vacate the trash.
                            len = 1024 * 4;
                            byte[] buf = new byte[len];
                            inEndpoint.Abort();
                            inEndpoint.Reset();
                            inEndpoint.XferData(ref buf, ref len);
                            Thread.Sleep(100);
                        }

                        Thread.Sleep(100);
                        bRunning = false;
                    }
                }

                inEndpoint.Abort();
                bRelease = true;
            }
            catch (Exception ex)
            {
                bRunning = false;
                bRelease = true;
            }
        }

        public void StatusUpdate()
        {
        }

        public void ImageView()
        {
            Bitmap bitmap = this.CreateBitmap(1024, 768, PixelFormat.Format16bppRgb565);
            bitmap = Utilities.RotateImage(bitmap, new PointF((float)(this.DatumPos_sX + this.CropSize_X / 2), (float)(this.DatumPos_sY + this.CropSize_Y / 2)), (byte)this.AngleInfo);
            Rectangle rect = new Rectangle(this.DatumPos_sX, this.DatumPos_sY, this.CropSize_X, this.CropSize_Y);
            this.CroppedImage = bitmap.Clone(rect, PixelFormat.Format16bppRgb565);
            this.CroppedImage.Save(Path.Combine(tempFolderPath, "LicenseImage.bmp"));
            bitmap.Dispose();

            this.picImage.ImageLocation = Path.Combine(tempFolderPath, "LicenseImage.bmp");
            this.picImage.Image = Image.FromFile(Path.Combine(tempFolderPath, "LicenseImage.bmp"));

            AppContext.Instance().Content.CurrentImageContent = AppContext.Instance().Content.ImageContent;
            AppContext.Instance().Content.PreviousContent = string.Empty;
            AppContext.Instance().Content.SetContent();
            timClearFields.Enabled = true;
            AppContext.Instance().Content.IsImage = "1";
        }

        public unsafe Bitmap CreateBitmap(int Width, int Height, PixelFormat Pfmt)
        {
            Bitmap result;
            if (Pfmt == PixelFormat.Format24bppRgb)
            {
                if (Width * Height * 3 != this.TotalPixSize)
                {
                    result = null;
                    return result;
                }
            }
            else
            {
                if (Pfmt != PixelFormat.Format16bppRgb565)
                {
                    result = null;
                    return result;
                }
                if (Width * Height * 2 != this.TotalPixSize)
                {
                    result = null;
                    return result;
                }
            }

            try
            {
                Bitmap bitmap = new Bitmap(Width, Height, Pfmt);
                BitmapData bitmapData = bitmap.LockBits(new Rectangle(0, 0, Width, Height), ImageLockMode.WriteOnly, Pfmt);
                byte* ptr = (byte*)bitmapData.Scan0.ToPointer();
                for (int i = 0; i < this.TotalPixSize / this.BufSz; i++)
                {
                    for (int j = 0; j < this.BufSz; j++)
                    {
                        if (j % 2 == 1)
                        {
                            *ptr = DataBuf[i][j];
                            ptr++;
                            *ptr = DataBuf[i][j - 1];
                            ptr++;
                        }
                    }
                }
                bitmap.UnlockBits(bitmapData);
                result = bitmap;
            }
            catch (Exception ex)
            {
                result = null;
            }
            return result;
        }

        public void ThreadException()
        {
            this.bRunning = false;
            this.tStartCap = null;
            this.timCheckStatus.Dispose();
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            try
            {
                DialogResult result = openFileDialog.ShowDialog();
                if (result == System.Windows.Forms.DialogResult.OK)
                {
                    picBackImage.ImageLocation = openFileDialog.FileName;
                    picImage.Image = Image.FromFile(openFileDialog.FileName);
                    _imageName = openFileDialog.SafeFileName;
                    ClearFields(false);
                    txtBarcode.Clear();
                    AppContext.Instance().Content.CurrentContent = AppContext.Instance().Content.EmptyContent;
                    AppContext.Instance().Content.CurrentImageContent = AppContext.Instance().Content.ImageContent;
                    AppContext.Instance().Content.PreviousContent = string.Empty;
                    AppContext.Instance().Content.IsImage = "1";
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void ClearFields(bool clearImage)
        {
            txtTitle.Text = string.Empty;
            txtFirstName.Text = string.Empty;
            txtMidName.Text = string.Empty;
            txtLastName.Text = string.Empty;
            txtAddress.Text = string.Empty;
            txtCity.Text = string.Empty;
            txtState.Text = string.Empty;
            txtZip.Text = string.Empty;
            txtDOB.Text = string.Empty;
            txtEXP.Text = string.Empty;
            txtHt.Text = string.Empty;
            txtWt.Text = string.Empty;
            txtSex.Text = string.Empty;
            txtHair.Text = string.Empty;
            txtEye.Text = string.Empty;
            txtClass.Text = string.Empty;
            txtRestrict.Text = string.Empty;
            txtEndors.Text = string.Empty;
            txtDobMMDD.Text = string.Empty;
            txtDobYear.Text = string.Empty;
            txtID.Text = string.Empty;
            txtRank.Text = string.Empty;
            txtPassportNumber.Text = string.Empty;
            txtNationality.Text = string.Empty;
            txtIssueDate.Text = string.Empty;

            if (!_isEseek && !clearImage)
            {

                AppContext.Instance().Content.CurrentImageContent = AppContext.Instance().Content.ImageContent;
                AppContext.Instance().Content.CurrentFaceImageContent = AppContext.Instance().Content.FaceImageContent;
                AppContext.Instance().Content.CurrentSignatureImageContent = AppContext.Instance().Content.SignatureImageContent;
                AppContext.Instance().Content.PreviousContent = string.Empty;
                AppContext.Instance().Content.IsImage = "1";
            }

            if (clearImage)
            {
                ClearImage();
            }
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            ClearFields(true);
            txtBarcode.Clear();
            AppContext.Instance().Content.CurrentContent = AppContext.Instance().Content.EmptyContent;
            AppContext.Instance().Content.IsImage = string.Empty;

            if (_idDataEx != null)
            {
                _idDataEx.ResetIDFields();
            }
        }

        private void toolStripMenuItemShow_Click(object sender, EventArgs e)
        {
            this.WindowState = FormWindowState.Normal;
            this.Show();
        }

        private void OpenPort()
        {
            this.labState.BackColor = Color.Red;

            if (this.serPort == null)
            {
                return;
            }

            if (this.serPort.IsOpen)
            {
                this.serPort.Close();
            }

            string port = ConfigurationManager.AppSettings["Port"].ToString();

            bool found = false;
            foreach (COMPortInfo comPort in COMPortInfo.GetCOMPortsInfo())
            {
                if (comPort.Name.Equals(port))
                {
                    found = true;
                    break;
                }
            }

            if (!found)
            {
                MessageBox.Show(port + " port wasn't found!");
                return;
            }

            string parityValue = ConfigurationManager.AppSettings["Parity"].ToString();
            string stopBit = ConfigurationManager.AppSettings["StopBit"].ToString();
            string flowControl = ConfigurationManager.AppSettings["FlowControl"].ToString();
            string baudRate = ConfigurationManager.AppSettings["BaudRate"].ToString();
            string dataBits = ConfigurationManager.AppSettings["DataBits"].ToString();

            if (!string.IsNullOrEmpty(port))
            {
                Parity parity = Parity.Space;
                switch (parityValue)
                {
                    case "None":
                        parity = Parity.None;
                        break;
                    case "Odd":
                        parity = Parity.Odd;
                        break;
                    case "Even":
                        parity = Parity.Even;
                        break;
                    case "Mark":
                        parity = Parity.Mark;
                        break;
                }

                StopBits stopBits = StopBits.None;
                this.labStopBit.Text = "NONE";

                switch (stopBit)
                {
                    case "1":
                        stopBits = StopBits.One;
                        this.labStopBit.Text = "ONE";
                        break;
                    case "1.5":
                        stopBits = StopBits.OnePointFive;
                        this.labStopBit.Text = "ONE.FIVE";
                        break;
                    case "2":
                        stopBits = StopBits.Two;
                        this.labStopBit.Text = "TWO";
                        break;
                }

                Handshake handshake = Handshake.None;
                this.labFlowCtr.Text = "None";

                switch (flowControl)
                {
                    case "CTS/RTS":
                        handshake = Handshake.RequestToSend;
                        this.labFlowCtr.Text = "RequestToSend";
                        break;
                    case "XOnXOff":
                        handshake = Handshake.XOnXOff;
                        this.labFlowCtr.Text = "XOnXOff";
                        break;
                }

                RegistryKey registryKey = Registry.CurrentUser.OpenSubKey("Software", true);
                RegistryKey registryKey2 = registryKey.OpenSubKey("M280DEMO", true);
                if (registryKey2 != null)
                {
                    registryKey2.SetValue("Port", port);
                    registryKey2.SetValue("Baud", int.Parse(baudRate));
                    registryKey2.SetValue("DataBits", int.Parse(dataBits));
                    registryKey2.SetValue("Parity", (int)parity);
                    registryKey2.SetValue("Flow", (int)handshake);
                    registryKey2.SetValue("Stop", (int)stopBits);
                    registryKey2.Close();
                }
                this.serPort.PortName = port;
                this.labCOM.Text = port;
                this.serPort.BaudRate = int.Parse(baudRate);
                this.labBaud.Text = baudRate;
                this.serPort.DataBits = int.Parse(dataBits);
                this.labDataBit.Text = dataBits;
                this.serPort.StopBits = stopBits;
                this.serPort.Parity = parity;
                this.serPort.Handshake = handshake;
                this.serPort.Open();

                ConfigureScanner();

                this.labState.Text = "Connected";
                this.labState.BackColor = Color.YellowGreen;
            }
        }

        private void serPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                // Obtain the number of bytes waiting in the port's buffer
                int bytes = serPort.BytesToRead;

                // Create a byte array buffer to hold the incoming data
                byte[] buffer = new byte[bytes];

                // Read the data from the port and store it in our buffer
                serPort.Read(buffer, 0, bytes);
                ReaderBuffMgr.DataBuilder(buffer, Environment.TickCount);
                d1 = DateTime.Now; //update timestamp

                //this will catch the GetInfo sent when we open the com port using the M430 programed as UAB COM port Emulation
                if ((ReaderBuffMgr.RawBytes[ss1] == 0x46) && (ReaderBuffMgr.RawBytes[ss2] == 0x69) && (ReaderBuffMgr.RawBytes[ss3] == 0x72) && (ReaderBuffMgr.RawBytes[MBD] == 0x6D) && (ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng - 1] == 0x06))
                {
                    Array.Resize(ref UFCOMbuff, ReaderBuffMgr.PrevLng); //You MUST resize the global to pass only current data
                    ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng] = 0;//insert null to create printable string
                    Array.Copy(ReaderBuffMgr.RawBytes, 0, UFCOMbuff, 0, ReaderBuffMgr.PrevLng);

                    ReaderBuffMgr.PrevLng = 0;
                    ReaderBuffMgr.count = 0;

                   // displaystring = To_ASCII_Str_Null2Space(UFCOMbuff);
                   // this.Invoke(new EventHandler(SetText));
                    bool ok = UfComScan.UfComInfo.Initialize(UFCOMbuff);
                    if (ok)
                    {
                        displayString = "UFCOM Initialized OK";
                        this.Invoke(new EventHandler(SetText));
                    }
                }

                fields = new Dictionary<string, string>();

                //test for a complete scan before calling pdf417 parsing
                if ((ReaderBuffMgr.RawBytes[ss1] == 0x01) && (ReaderBuffMgr.RawBytes[ss2] == 0x02) && (ReaderBuffMgr.RawBytes[ss3] == 0x5D) && (ReaderBuffMgr.RawBytes[MBD] == 0x4C) && (ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng - 1] == 0x04))
                {
                    Array.Resize(ref buff, ReaderBuffMgr.PrevLng); //You MUST resize the global to pass only current data
                    ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng] = 0;//insert null to create printable string
                    Array.Copy(ReaderBuffMgr.RawBytes, 0, buff, 0, ReaderBuffMgr.PrevLng);

                    ReaderBuffMgr.PrevLng = 0; //reset conters
                    ReaderBuffMgr.count = 0;  //for next scan

                    //the following code supports data logging to a file
                    Array.Resize(ref Rawbuff, buff.Length - ss1);
                    Array.Copy(buff, ss1, Rawbuff, 0, buff.Length - ss1);
                    rawdata = ByteArrayToHexString(Rawbuff); //convert binary to hex string
                    ASCIIdata = new ASCIIEncoding().GetString(Rawbuff);
                    ASCIIdata = string.Concat(ASCIIdata, d1.ToString());

                    //end of data logging

                    _p = new Parser();

                    try
                    {
                        fields = _p.PDF417Process(Rawbuff); //
                    }
                    catch
                    {
                        MessageBox.Show("Invalid Barcode");
                    }
                    //displayString = PDFDLL.Barcode.PDF_String;
                    //this.Invoke(new EventHandler(SetText));

                    if (this.IsHandleCreated)
                    {
                        this.Invoke(new EventHandler(SetFields));
                    }
                }

                //test for magstripe read from scanner
                if ((ReaderBuffMgr.RawBytes[ss1] == 01) && (ReaderBuffMgr.RawBytes[ss2] == 02) && (ReaderBuffMgr.RawBytes[ss3] == 0x5D) && (ReaderBuffMgr.RawBytes[MBD] == 0x4D) && (ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng - 1] == 0x04))
                {
                    Array.Resize(ref MagBuff, ReaderBuffMgr.PrevLng); //You MUST resize the global to pass only current data
                    ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng] = 0;//insert null to create printable string
                    Array.Copy(ReaderBuffMgr.RawBytes, 0, MagBuff, 0, ReaderBuffMgr.PrevLng);

                    ReaderBuffMgr.PrevLng = 0; //reset counters
                    ReaderBuffMgr.count = 0; //for next swipe

                    //the following code supports data logging to a file
                    Array.Resize(ref Rawbuff, MagBuff.Length - ss1);
                    Array.Copy(MagBuff, ss1, Rawbuff, 0, MagBuff.Length - ss1);
                    rawdata = ByteArrayToHexString(Rawbuff); //convert binary to hex string
                    ASCIIdata = new ASCIIEncoding().GetString(Rawbuff);
                    ASCIIdata = string.Concat(ASCIIdata, d1.ToString());
                    //end of data logging

                    try
                    {
                        _p = new Parser();
                        fields = _p.MagProcess(MagBuff);
                    }
                    catch
                    {
                        MessageBox.Show("Invalid Magstripe");
                    }
                    //displayString = MAGDLL.Magstripe.track1 + MAGDLL.Magstripe.track2 + MAGDLL.Magstripe.track3;
                    //this.Invoke(new EventHandler(SetText));

                    if (this.IsHandleCreated)
                    {
                        this.Invoke(new EventHandler(SetFields));
                    }
                }

                
                //this will catch bad barcode reads
                if ((ReaderBuffMgr.RawBytes[ss1] == 01) && (ReaderBuffMgr.RawBytes[ss2] == 02) && (ReaderBuffMgr.RawBytes[ss3] == 0x4E) && (ReaderBuffMgr.RawBytes[MBD] == 0x52) && (ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng - 1] == 0x04))
                {
                    ReaderBuffMgr.PrevLng = 0;
                    ReaderBuffMgr.count = 0;

                    displayString = "\r\n Bad Barcode Read, Try Again";
                    //this.Invoke(new EventHandler(SetText));

                }

                //this catches the ACK from the Ping command
                if ((ReaderBuffMgr.RawBytes[0] == 0x06) && (ReaderBuffMgr.PrevLng == 0x01))
                {
                    ReaderBuffMgr.PrevLng = 0;
                    ReaderBuffMgr.count = 0;
                    // Show the user the incoming data in hex format
                    displayString = "ACK!";
                    //this.Invoke(new EventHandler(SetText));
                }
            }
            catch (Exception ex)
            {
            }
        }

        public string ByteArrayToHexString(byte[] data)
        {
            StringBuilder sb = new StringBuilder(data.Length * 3);
            foreach (byte b in data)
                sb.Append(Convert.ToString(b, 16).PadLeft(2, '0').PadRight(3, ' '));
            return sb.ToString().ToUpper();
        }

        private void SetText(object o, EventArgs e)
        {
            ClearImage();
            txtBarcode.Text = displayString;
        }

        private void SetFields(object o, EventArgs e)
        {
            ClearImage();
            if ((fields != null) && (fields.Count > 0))
            {
                AppContext.Instance().Content.IsImage = "0";
                txtState.Text = fields["State"];
                txtCity.Text = fields["City"];
                txtAddress.Text = fields["Address"];
                txtFirstName.Text = fields["FirstName"];
                txtMidName.Text = fields["MidName"];
                txtLastName.Text = fields["LastName"];
                txtTitle.Text = fields["Title"];
                txtZip.Text = fields["Zipcode"];
                if (txtZip.Text.Length > 5)
                {
                    txtZip.Text = txtZip.Text.Substring(0, 5);
                }

                txtDOB.Text = fields["DOB"];
                txtDobMMDD.Text = fields["DOBMMDD"];
                txtDobYear.Text = fields["DOBYEAR"];
                txtEXP.Text = fields["Exp"];
                txtHt.Text = fields["Ht"];
                txtWt.Text = fields["Wt"];
                txtSex.Text = fields["Sex"];
                txtHair.Text = fields["Hair"];
                txtEye.Text = fields["Eye"];
                txtClass.Text = fields["Class"];
                txtRestrict.Text = fields["Restrict"];
                txtEndors.Text = fields["Endors"];
                txtID.Text = fields["ID"];
                txtRank.Text = fields["Rank"];

                AppContext.Instance().Content.SetContent();
                AppContext.Instance().Content.CurrentImageContent = string.Empty;
                AppContext.Instance().Content.PreviousContent = string.Empty;
                timClearFields.Enabled = true;
            }
        }

        public byte[] HexStringToByteArray(string s)
        {
            s = s.Replace(" ", "");
            byte[] buffer = new byte[s.Length / 2];
            for (int i = 0; i < s.Length; i += 2)
                buffer[i / 2] = (byte)Convert.ToByte(s.Substring(i, 2), 16);
            return buffer;
        }

        private void btnClrTxt_Click(object sender, EventArgs e)
        {
            this.txtBarcode.Text = "";
        }

        private void picImage_MouseMove(object sender, MouseEventArgs e)
        {
            this.labXpos.Text = e.X.ToString();
            this.labYpos.Text = e.Y.ToString();
            if (this.CroppedImage != null && e.Y < this.CroppedImage.Size.Height && e.X < this.CroppedImage.Size.Width)
            {
                Color pixel = this.CroppedImage.GetPixel(e.X, e.Y);
                this.labR.Text = pixel.R.ToString();
                this.labG.Text = pixel.G.ToString();
                this.labB.Text = pixel.B.ToString();
            }
        }

        private void btnCapture_Click(object sender, EventArgs e)
        {
            try
            {
                ClearFields(true);
                txtBarcode.Clear();

                AppContext.Instance().Content.CurrentContent = string.Empty;
                AppContext.Instance().Content.IsImage = "0";

                if (!_isEseek)
                {
                    this.Text = "Eyemetric Tray     Mode: " + ConfigurationManager.AppSettings["ScannerType"].ToString();

                    if (!_isEseek)
                    {
                        string scanner = CSSNConstants.GetScannerNameByType(_sLib.ScannerType);
                        if (!string.IsNullOrEmpty(scanner))
                        {
                            this.Text += "    Type: " + scanner;
                        }
                    }

                    _ScanDocument();
                    ExtractData();

                    AppContext.Instance().Content.SetContent();
                    AppContext.Instance().Content.CurrentImageContent = string.Empty;
                    AppContext.Instance().Content.PreviousContent = string.Empty;
                    timClearFields.Enabled = true;
                }
                else
                {
                    if (this.MyDevice == null || this.ep0 == null || this.inEndpoint == null)
                    {
                        MessageBox.Show("Please Connect M280 Scanner to PC");
                        return;
                    }

                    //|| this.bInitM280 == false
                    if (this.bRunning == true || this.bSysBusy == true)
                    {
                        return;
                    }

                    if (bRelease == false)
                    {
                        MessageBox.Show("bRelease is false");
                        return;
                    }
                    bRelease = false;

                    bRunning = true;

                    BufSz = inEndpoint.MaxPktSize * 24;
                    QueueSz = BufNum;

                    inEndpoint.XferSize = BufSz;

                    // Initializing Data Buffer
                    DataBuf = new byte[TotalPixSize / BufSz][];
                    for (int i = 0; i < (TotalPixSize / BufSz); i++)
                    {
                        DataBuf[i] = new byte[BufSz];
                    }

                    tStartCap = new Thread(new ThreadStart(XferThread));
                    tStartCap.IsBackground = true;
                    tStartCap.Priority = ThreadPriority.Highest;
                    tStartCap.Start();
                    // Send Start capture command to M280
                    SendCommand(218, 0, 0);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("btnCapture_Click : " + ex.Message);
            }
        }

        private void _ScanDocument()
        {
            if (_sLib.IsScannerValid == 1)
            {
                _sLib.Resolution = 300;
                _sLib.ScanHeight = -1;
                _sLib.ScanWidth = -1;

                int result;
                // Scan to file
                result = _sLib.ScanToFileEx(filePath);

                if (result < 0)
                {
                    CSSNUtilities.Alert(CSSNCore.GetScannerClassError(result));
                    return;
                }

                this.picImage.Image = CSSNUtilities._GetImageFromFile(filePath);
            }
            else
            {
                MessageBox.Show("Scanner not valid. Please check the connection to the scanner.");
            }
        }

        private void ExtractData()
        {
            int result = -1;
            int stateID = -1; // State not found yet


            int angleA = 0;
            int angleB = 0;
            int imageA_assignment = 0;

            result = _passport.Process(string.Empty);

            if (result >= 0)
            {
                txtFirstName.Text = _passport.NameFirst;
                if (string.IsNullOrEmpty(_passport.NameFirst))
                {
                    txtFirstName.Text = _passportEx.NameFirstNoneMRZ;
                }
                txtMidName.Text = _passport.NameMiddle;
                txtLastName.Text = _passport.NameLast;
                if (string.IsNullOrEmpty(_passport.NameLast))
                {
                    txtLastName.Text = _passportEx.NameLastNoneMRZ;
                }
                txtID.Text = _passport.PersonalNumber;
                //txtDOB.Text = _passport.DateOfBirth;
                if (!string.IsNullOrEmpty(_passport.DateOfBirth))
                {
                    string[] d = _passport.DateOfBirth.Split(new string[] { "-" }, StringSplitOptions.RemoveEmptyEntries);
                    if (d.Length >= 3)
                    {
                        txtDobMMDD.Text = d[0] + d[1];
                        txtDobYear.Text = d[2];
                    }
                }
                txtEXP.Text = _passport.ExpirationDate;
                txtCity.Text = _passportEx.End_POB;

                txtState.Text = _passport.country;
                txtSex.Text = _passport.Sex;

                txtPassportNumber.Text = _passport.PassportNumber;

                if (string.IsNullOrEmpty(_passport.PassportNumber))
                {
                    txtPassportNumber.Text = _passportEx.PassPassportNum_NON_MRZ;
                }

                txtIssueDate.Text = _passportEx.IssueDate;
                txtEXP.Text = _passport.ExpirationDate;
                txtNationality.Text = _passport.Nationality;

                result = _passport.GetFace(_faceImage);
                if (result > 0)
                {
                    picFaceImage.Image = CSSNUtilities._GetImageFromFile(_faceImage);
                }
                else
                {
                    picFaceImage.Image = null;
                }

                result = _passportEx.GetSignature(string.Empty, _signImage);
                if (result > 0)
                {
                    picSign.Image = CSSNUtilities._GetImageFromFile(_signImage);
                }
                else
                {
                    picSign.Image = null;
                }
            }
            else
            {

                string scanner = CSSNConstants.GetScannerNameByType(_sLib.ScannerType);
                if ((scanner.ToLower().Contains("800dxn") || scanner.ToLower().Contains("800dx") || scanner.ToLower().Contains("duplex") || scanner.ToLower().Contains("passport")))
                {

                    stateID = _idDataEx.DetectProcessDuplexEX(string.Empty, string.Empty, stateID, ref imageA_assignment, ref angleA, ref angleB, 0);
                }

                if (stateID < 0)
                {
                    stateID = _idData.AutoDetectState(string.Empty);
                }

                //if (stateID < 0)
                //{
                //    isBarcodeRead = false;
                //    isOCRRead = false;
                //    CSSNUtilities.Alert(CSSNCore.GetIdDataClassError(result));
                //    return;
                //}

                _idData.RefreshData();
                _barCodeEx.Using4DigitsYearFormat = 1;
                _barCode.RefreshData();



                string rawData;
                _barCode.GetRawData(out rawData);

                isBarcodeRead = !string.IsNullOrEmpty(rawData);
                isOCRRead = true;

                GetData(rawData);

                result = _barCodeEx.Proc1DImage("");
                if (result >= 1)
                {
                    txtBarcode.Text = _barCodeEx.Data1D;
                }

                result = _idData.GetFaceImage(string.Empty, _faceImage, stateID);
                if (result > 0)
                {
                    picFaceImage.Image = CSSNUtilities._GetImageFromFile(_faceImage);
                }

                // Get Signature image
                result = _idData.GetSignImage(string.Empty, _signImage, stateID);
                if (result > 0)
                {
                    picSign.Image = CSSNUtilities._GetImageFromFile(_signImage);
                }

                _DumpImage();
            }
        }

        private void GetData(string barcode)
        {
            if (isBarcodeRead)
            {
                if (_useAcuantBarcodeParser)
                {
                    if (!string.IsNullOrEmpty(_barCode.license))
                    {
                        txtFirstName.Text = _barCode.NameFirst;
                        txtMidName.Text = _barCode.NameMiddle;
                        txtLastName.Text = _barCode.NameLast;
                        txtTitle.Text = _barCode.NameSuffix;
                        txtID.Text = _barCode.license;
                        txtDOB.Text = _barCode.DateOfBirth;
                        if (!string.IsNullOrEmpty(_barCode.DateOfBirth))
                        {
                            string[] d = _barCode.DateOfBirth.Split(new string[] { "-" }, StringSplitOptions.RemoveEmptyEntries);
                            if (d.Length >= 3)
                            {
                                txtDobMMDD.Text = d[0] + d[1];
                                txtDobYear.Text = d[2];
                            }
                        }
                        txtEXP.Text = _barCode.ExpirationDate;
                        txtAddress.Text = _barCode.Address;
                        txtCity.Text = _barCode.City;
                        txtState.Text = _barCode.State;
                        txtZip.Text = _barCode.Zip;
                        txtSex.Text = _barCode.Sex;
                        txtClass.Text = _barCode.Class;
                        txtEye.Text = _barCodeEx.Eyes;
                        txtHair.Text = _barCodeEx.Hair;
                        txtHt.Text = _barCodeEx.Height;
                        txtWt.Text = _barCodeEx.Weight;
                    }
                    else
                    {
                        txtFirstName.Text = _idData.NameFirst;
                        txtMidName.Text = _idData.NameMiddle;
                        txtLastName.Text = _idData.NameLast;
                        txtTitle.Text = _idData.NameSuffix;
                        txtID.Text = _idData.license;
                        txtDOB.Text = _idDataEx.DateOfBirth4;
                        if (!string.IsNullOrEmpty(_idDataEx.DateOfBirth4))
                        {
                            string[] d = _idDataEx.DateOfBirth4.Split(new string[] { "-" }, StringSplitOptions.RemoveEmptyEntries);
                            if (d.Length >= 3)
                            {
                                txtDobMMDD.Text = d[0] + d[1];
                                txtDobYear.Text = d[2];
                            }
                        }
                        txtEXP.Text = _idDataEx.ExpirationDate4;
                        txtAddress.Text = _idData.Address;
                        txtCity.Text = _idData.City;
                        txtState.Text = _idData.State;
                        txtZip.Text = _idData.Zip;
                        txtSex.Text = _idData.Sex;
                        txtClass.Text = _idData.Class;
                        txtEye.Text = _idData.Eyes;
                        txtHair.Text = _idData.Hair;
                        txtHt.Text = _idData.Height;
                        txtWt.Text = _idData.Weight;

                        txtRestrict.Text = _idData.Restriction;
                        txtEndors.Text = _idData.Endorsements;
                    }
                }
                else
                {
                    byte[] input = HexStringToByteArray(GetHex(barcode));
                   // Array.Resize(ref MagBuff, input.Length);
                    Array.Resize(ref buff, input.Length);
                    Array.Copy(input, buff, input.Length);
                    int ret = Barcode.Parse417(input);

                    txtFirstName.Text = Barcode.First_DL;
                    txtMidName.Text = Barcode.Mid_DL;
                    txtLastName.Text = Barcode.Last_DL;
                    txtTitle.Text = Barcode.Title_DL;
                    txtID.Text = Barcode.Dl_DL;
                    txtDOB.Text = Barcode.DobMMDD_DL + Barcode.DobYear_DL;
                    txtDobMMDD.Text = Barcode.DobMMDD_DL;
                    txtDobYear.Text = Barcode.DobYear_DL;
                    txtEXP.Text = Barcode.Exp_DL;
                    txtAddress.Text = Barcode.Addr_DL;
                    txtCity.Text = Barcode.City_DL;
                    txtState.Text = Barcode.State_DL;
                    txtZip.Text = Barcode.Zip_DL;
                    txtSex.Text = Barcode.Sex_DL;
                    txtClass.Text = Barcode.Class_DL;
                    txtEye.Text = Barcode.Eye_DL;
                    txtHair.Text = Barcode.Hair_DL;
                    txtHt.Text = Barcode.Height_DL;
                    txtWt.Text = Barcode.Wt_DL;

                    txtRestrict.Text = Barcode.Rest_DL;
                    txtEndors.Text = Barcode.Endr_DL;
                    txtRank.Text = Barcode.Rank_DL;
                }
            }
            else
            {
                txtFirstName.Text = _idData.NameFirst;
                txtMidName.Text = _idData.NameMiddle;
                txtLastName.Text = _idData.NameLast;
                txtTitle.Text = _idData.NameSuffix;
                txtID.Text = _idData.license;
                txtDOB.Text = _idDataEx.DateOfBirth4;
                if (!string.IsNullOrEmpty(_idDataEx.DateOfBirth4))
                {
                    string[] d = _idDataEx.DateOfBirth4.Split(new string[] { "-" }, StringSplitOptions.RemoveEmptyEntries);
                    if (d.Length >= 3)
                    {
                        txtDobMMDD.Text = d[0] + d[1];
                        txtDobYear.Text = d[2];
                    }
                }
                txtEXP.Text = _idDataEx.ExpirationDate4;
                txtAddress.Text = _idData.Address;
                txtCity.Text = _idData.City;
                txtState.Text = _idData.State;
                txtZip.Text = _idData.Zip;
                txtSex.Text = _idData.Sex;
                txtClass.Text = _idData.Class;
                txtEye.Text = _idData.Eyes;
                txtHair.Text = _idData.Hair;
                txtHt.Text = _idData.Height;
                txtWt.Text = _idData.Weight;

                txtRestrict.Text = _idData.Restriction;
                txtEndors.Text = _idData.Endorsements;
            }
        }

        private void _DumpImage()
        {
            int result = _image.ReformatImage(string.Empty, 0, 0, filePath);

            result = _image.ReformatImage(string.Empty, 0, 0, filePath);
            _imageEx.SetMainImage(2);
            int result1 = _image.ReformatImage(string.Empty, 0, 0, filePath_back);
            _imageEx.SetMainImage(0);

            if (result > -1)
            {
                this.picImage.Image = CSSNUtilities._GetImageFromFile(filePath);
            }

            if (result1 > -1)
            {
                this.picBackImage.Image = CSSNUtilities._GetImageFromFile(filePath_back);
            }
        }

        #region IContent

        public string Title
        {
            get
            {
                return txtTitle.Text;
            }
        }

        public string FirstName
        {
            get
            {
                return txtFirstName.Text;
            }
        }

        public string MiddleName
        {
            get
            {
                return txtMidName.Text;
            }
        }

        public string LastName
        {
            get
            {
                return txtLastName.Text;
            }
        }

        public string Address
        {
            get
            {
                return txtAddress.Text;
            }
        }

        public string City
        {
            get
            {
                return txtCity.Text;
            }
        }

        public string State
        {
            get
            {
                return txtState.Text;
            }
        }

        public string Zip
        {
            get
            {
                return txtZip.Text;
            }
        }

        public string DOB
        {
            get
            {
                return txtDOB.Text;
            }
        }

        public string EXP
        {
            get
            {
                return txtEXP.Text;
            }
        }

        public string HeightValue
        {
            get
            {
                return txtHt.Text;
            }
        }

        public string Weight
        {
            get
            {
                return txtWt.Text;
            }
        }

        public string Sex
        {
            get
            {
                return txtSex.Text;
            }
        }

        public string Hair
        {
            get
            {
                return txtHair.Text;
            }
        }

        public string Eye
        {
            get
            {
                return txtEye.Text;
            }
        }

        public string Class
        {
            get
            {
                return txtClass.Text;
            }
        }

        public string Restrict
        {
            get
            {
                return txtRestrict.Text;
            }
        }

        public string Endors
        {
            get
            {
                return txtEndors.Text;
            }
        }

        public string DobMMDD
        {
            get
            {
                return txtDobMMDD.Text;
            }
        }

        public string DobYear
        {
            get
            {
                return txtDobYear.Text;
            }
        }

        public string ID
        {
            get
            {
                return txtID.Text;
            }
        }

        public string Rank
        {
            get
            {
                return txtRank.Text;
            }
        }

        public string BarcodeValue
        {
            get
            {
                return txtBarcode.Text;
            }
        }


        public string ImageContent
        {
            get
            {
                string content = string.Empty;
                if ((!string.IsNullOrEmpty(txtBarcode.Text)) && (picBackImage.Image != null))
                {
                    txtBarcode.Text = string.Empty;
                    using (MemoryStream ms = new MemoryStream())
                    {
                        picBackImage.Image.Save(ms, ImageFormat.Png);
                        content = @"data:image/png;base64," + Convert.ToBase64String(ms.ToArray());
                    }
                }
                else if (picImage.Image != null)
                {
                    using (MemoryStream ms = new MemoryStream())
                    {
                        picImage.Image.Save(ms, ImageFormat.Png);
                        content = @"data:image/png;base64," + Convert.ToBase64String(ms.ToArray());
                    }
                }
                picBackImage.Image = null;
                picImage.Image = null;
                return content;
            }
        }

        public string FaceImageContent
        {
            get
            {
                string content = string.Empty;
                if (picFaceImage.Image != null)
                {
                    using (MemoryStream ms = new MemoryStream())
                    {
                        Bitmap b = new Bitmap(picFaceImage.Image);
                        b.Save(ms, ImageFormat.Png);
                        content = @"data:image/png;base64," + Convert.ToBase64String(ms.ToArray());
                    }
                }
                picFaceImage.Image = null;
                return content;
            }
        }

        public string SignatureImageContent
        {
            get
            {
                string content = string.Empty;
                if (picSign.Image != null)
                {
                    using (MemoryStream ms = new MemoryStream())
                    {
                        Bitmap b = new Bitmap(picSign.Image);
                        b.Save(ms, ImageFormat.Png);
                        content = @"data:image/png;base64," + Convert.ToBase64String(ms.ToArray());
                    }
                }
                picSign.Image = null;
                return content;
            }
        }

        public string Nationality
        {
            get
            {
                return txtNationality.Text;
            }
        }

        public string PassportNumber
        {
            get
            {
                return txtPassportNumber.Text;
            }
        }

        public string IssueDate
        {
            get
            {
                return txtIssueDate.Text;
            }
        }

        public string PreviousContent
        {
            get { return _previousContent; }
            set { _previousContent = value; }
        }

        public string CurrentContent
        {
            get { return _currentContent; }
            set { _currentContent = value; }
        }

        public string CurrentImageContent
        {
            get { return _currentImageContent; }
            set
            {
                _currentImageContent = value;
                if (string.IsNullOrEmpty(value))
                {
                    _currentFaceImageContent = string.Empty;
                    _currentSignatureImageContent = string.Empty;
                }
            }
        }

        public string CurrentFaceImageContent
        {
            get { return _currentFaceImageContent; }
            set { _currentFaceImageContent = value; }
        }

        public string CurrentSignatureImageContent
        {
            get { return _currentSignatureImageContent; }
            set { _currentSignatureImageContent = value; }
        }

        public void SetContent()
        {
            IContent ct = AppContext.Instance().Content;
            string sep = "##;";
            string result = ct.Title + sep + ct.FirstName + sep + ct.MiddleName + sep + ct.LastName + sep + ct.Address + sep;
            result += ct.City + sep + ct.State + sep + ct.Zip + sep + ct.DOB + sep + ct.EXP + sep + ct.HeightValue + sep + ct.Weight + sep;
            result += ct.Sex + sep + ct.Hair + sep + ct.Eye + sep + ct.Class + sep + ct.Restrict + sep + ct.Endors + sep + ct.DobMMDD + sep;
            result += ct.DobYear + sep + ct.ID + sep + ct.Rank;

            ct.CurrentContent = result;
        }

        public string EmptyContent
        {
            get
            {
                string result = string.Empty;
                for (int i = 0; i < 29; i++)
                {
                    result += "##;";
                }
                return result;
            }
        }

        public string IsImage
        {
            get
            {
                return _isImage;
            }
            set
            {
                _isImage = value;
            }
        }

        #endregion

        private void portSettingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                bool wasOpen = false;
                if (this.serPort.IsOpen)
                {
                    this.serPort.Close();
                    wasOpen = true;
                }

                PortSettings frmSettings = new PortSettings();
                DialogResult result = frmSettings.ShowDialog();
                if (result == System.Windows.Forms.DialogResult.OK)
                {
                    //Apply the new settings to the port
                    OpenPort();
                }
                else
                {
                    HandleServiceHost();

                    if (wasOpen)
                    {
                        this.serPort.Open();
                        ConfigureScanner();
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void btnRotate_Click(object sender, EventArgs e)
        {
            Image img = this.picImage.Image;
            if (img != null)
            {
                try
                {
                    Bitmap bmp = new Bitmap(img);
                    bmp.RotateFlip(RotateFlipType.Rotate90FlipNone);
                    this.picImage.Image = bmp;
                    img.Dispose();
                    img = null;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void TestForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.Hide();
            e.Cancel = true;
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                if (_sLibEx != null)
                {
                    _sLibEx.UnInit();
                }

                if (m_hPort != IntPtr.Zero)
                {
                    RegisterCallBack(false);

                    // Close Port
                    CPFW.cpfw_close(m_hPort);
                    m_hPort = IntPtr.Zero;
                }

                Environment.Exit(0);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void ConfigureScanner()
        {
            if (m_hPort != IntPtr.Zero)
            {
                string getInfo = "NLS0003000;";
                scannerInit = true;
                SendString(getInfo);
            }

            //this is for the Eseek, the UFCOM M430 does not use programming strings, it gets programmed via reading programming barcodes.
            if (serPort.IsOpen)
            {
                //    ////this sends out NLS0003000; to the M430 as and is used to unlock the parsing DLLs by reading the s/n and ESN
            byte[] data1 = HexStringToByteArray("4E 4C 53 30 30 30 33 30 30 30 3B"); // Ping Scaner to send the GetInfo command
            //    // Send the binary data out the port
            serPort.Write(data1, 0, data1.Length);
            }
        }

        private void timClearFields_Tick(object sender, EventArgs e)
        {
            Thread.Sleep(2000);
            ClearFields(false);
            txtBarcode.Clear();
            timClearFields.Enabled = false;
        }

        protected override void WndProc(ref System.Windows.Forms.Message m)
        {
            base.WndProc(ref m);

            if (m.Msg == WM_QUERYENDSESSION)
            {
                try
                {
                    if (_sLibEx != null)
                    {
                        _sLibEx.UnInit();
                    }

                    if (m_hPort != IntPtr.Zero)
                    {
                        RegisterCallBack(false);

                        // Close Port
                        CPFW.cpfw_close(m_hPort);
                        m_hPort = IntPtr.Zero;
                    }

                    Environment.Exit(0);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void ClearImage()
        {
            picSign.Image = null;
            picFaceImage.Image = null;

            Graphics graphics = this.picImage.CreateGraphics();
            graphics.Clear(Color.White);
            picImage.Image = null;
            this.CroppedImage = null;
            if (picBackImage.Image != null)
            {
                this.picBackImage.CreateGraphics();
                graphics.Clear(Color.White);
                picBackImage.Image = null;
            }
        }

        private void _InitSDK()
        {
            if (_sLib == null)
            {
                return;
            }

            _sLib.ScanHeight = -1;
            _sLib.ScanWidth = -1;

            _sLibEx.DefaultScanner = CSSNConstants.CSSN_800DX;
            _sLibEx.Duplex = 1;
            _sLib.Resolution = 600;

            // Initializing Scanner Library
            int result = _sLib.InitLibrary(CSSNCore.license);


            if (result < 0 && result != -13)
            {
                CSSNUtilities.Alert(CSSNCore.GetScannerClassError(result));
            }
            else
            {
                //  Initializing Image Library
                result = _image.InitLibrary(CSSNCore.license);

                // Initializing Id data library
                result = _idData.InitLibrary(CSSNCore.license);

                //Initializing barcode library
                result = _barCode.InitLibrary(CSSNCore.license);

                // Init Passport library
                result = _passport.Init(CSSNCore.license);

                //tsStatuslblScannerName.Text = CSSNConstants.GetScannerNameByType(_sLib.ScannerType); // Dispaly connected scanner name
            }
        }

        private void tmrCheckLicenseInsert_Tick(object sender, EventArgs e)
        {
            if (_sLib != null)
            {
                if (_sLib.PaperInTray == 1)
                {
                    btnCapture_Click(sender, EventArgs.Empty);
                }

                if (_sLib.PressedButton >= 1)
                {
                    btnCapture_Click(sender, EventArgs.Empty);
                }
            }
        }

        private void btnCalibrate_Click(object sender, EventArgs e)
        {
            if (_isEseek)
            {
                ConfigureScanner();
            }
            else if (_sLibEx != null)
            {
                _sLibEx.CalibrateScannerEx();
            }
        }

        private void chkDupplex_CheckedChanged(object sender, EventArgs e)
        {
            _InitSDK();
        }

        private void acuantToolStripMenuItem_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                eSeekToolStripMenuItem.CheckedChanged -= eSeekToolStripMenuItem_CheckedChanged;
                eSeekToolStripMenuItem.Checked = false;
                eSeekToolStripMenuItem.CheckedChanged += eSeekToolStripMenuItem_CheckedChanged;

                _isEseek = false;

                if (_loaded)
                {
                    HandleScanner();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void eSeekToolStripMenuItem_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                acuantToolStripMenuItem.CheckedChanged -= acuantToolStripMenuItem_CheckedChanged;
                acuantToolStripMenuItem.Checked = false;
                acuantToolStripMenuItem.CheckedChanged += acuantToolStripMenuItem_CheckedChanged;

                _isEseek = true;

                if (_loaded)
                {
                    HandleScanner();
                }
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                {
                    MessageBox.Show(ex.InnerException.Message);
                }
                else
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void acuantToolStripMenuItem_Click(object sender, EventArgs e)
        {
            acuantToolStripMenuItem.Checked = true;
        }

        private void eSeekToolStripMenuItem_Click(object sender, EventArgs e)
        {
            eSeekToolStripMenuItem.Checked = true;
        }

        private string GetHex(string str)
        {
            StringBuilder sb = new StringBuilder();
            byte[] inputBytes = Encoding.UTF8.GetBytes(str);
            foreach (byte b in inputBytes)
            {
                sb.Append(string.Format("{0:x2}", b));
            }
            return sb.ToString();
        }

        private void FtdiReload()
        {
            FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;
            FTDI myFtdiDevice = new FTDI();
            ushort wVID = 0x403;
            ushort wPID = 0x6001;
            try
            {
                ftStatus = myFtdiDevice.Reload(wVID, wPID);
                if (ftStatus != FTDI.FT_STATUS.FT_OK)
                    MessageBox.Show("FTDI Reload Failure");
            }
            catch
            {
                MessageBox.Show("Exception:FTDI Reload Failure!");
            }
        }


        #region M420 methods
        private void RegisterCallBack(bool register)
        {
            if (NTF_DATA_event == null)
                NTF_DATA_event = new CPFW.NTF_DATA_delegate(DataReceivedCallBack);

            if (register)
            {
                int iresult = CPFW.cpfw_register_notify(m_hPort, null, NTF_DATA_event, IntPtr.Zero);
                if (iresult != CPFW.ERR_SUCCEED)
                    ShowError("RegisterCallBack", iresult);
            }
            else
                CPFW.cpfw_unregister_notify(m_hPort, true, true);
        }

        private bool UpdateControls()
        {
            bool reslt = false;
            if (m_hPort != IntPtr.Zero)
            {
                int iresult = CPFW.cpfw_QueryDevStatus(m_hPort);
                if (iresult == CPFW.STATE_CONNECT)
                {
                    this.labUSBState.Text = "Connected";
                    this.labUSBState.BackColor = Color.YellowGreen;
                    reslt = TokenScanner.TkwkScan.Initialize();
                }
                else
                    ShowError("STATE_CONNECT", iresult);
            }
            return reslt;
        }

        private void ShowError(string funcName, int code)
        {
            StringBuilder strErr = new StringBuilder(512);

            if (CPFW.cpfw_translate_err_code(code, strErr, strErr.Capacity + 1) == CPFW.ERR_SUCCEED)
                msg = funcName + " fails -> " + strErr;
            else
                msg = funcName + " fails -> error code: " + code;

            MessageBox.Show(msg);
        }


        private int SendString(string str2Send)
        {
            Encoding enc = Encoding.GetEncoding(1252);
            byte[] srcBytes = Encoding.Unicode.GetBytes(str2Send);
            byte[] dstBytes = Encoding.Convert(Encoding.Unicode, enc, srcBytes);
            return SendBytes(dstBytes);
        }

        private int SendBytes(byte[] bytes)
        {
            int len = bytes.Length;
            //
            IntPtr ptrBuf = Marshal.AllocHGlobal(len);
            Marshal.Copy(bytes, 0, ptrBuf, len);
            //
            int iresult = CPFW.cpfw_write(m_hPort, ptrBuf, len, 200);
            //
            Marshal.FreeHGlobal(ptrBuf);
            //
            if (iresult < 0)
            {
                //ShowError("CPFW.cpfw_write", iresult);
            }
            else if (iresult == 0)
            {
                MessageBox.Show("TimeOut Error", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return iresult;
        }

        public static string To_ASCII_Str_Null2Space(byte[] bytes)
        {
            string hexString = "";
            for (int i = 0; i < bytes.Length; i++)
            {
                if (bytes[i] == 0) bytes[i] = 0x20; //this should keep nulls out of strings
                hexString += (char)bytes[i];

            }
            return hexString;
        }

        #endregion

        #region M420 DataRecvCallBack
        private void DataReceivedCallBack(IntPtr pBuf, int dwDataLen, IntPtr pParam)
        {
            try
            {
                if (scannerInit)
                {
                    scannerInit = false;

                    if (dwDataLen > 1)
                    {
                        //this sets every parameter for M420/H325 to work with Parsing DLLS 
                        //disables 1D barcodes
                        string TkWkCMD = "NLS0001010;NLS0501020;NLS0311010;NLS0317040;NLS0307010;NLS0308000;NLS0300000=0x01025D;NLS0310000=0x0304;NLS0005000=0x4C;NLS0004020=0x43;NLS0004130=0x43;NLS0201000;0000160;";

                        SendString(TkWkCMD);
                    }
                    else
                    {
                        //this sets every parameter for F406 to work with Parsing DLLS, 
                        string CMDF406 = "NLS0006010;NLS0001010;NLS0501020;NLS0313030;NLS0302000;NLS0311010;NLS0317040;NLS0307010;NLS0308000;NLS0300000=0x01025D;NLS0310000=0x0304;NLS0005000=0x4C;NLS0004020=0x43;NLS0004130=0x43;0000160;NLS0006000;";
                        SendString(CMDF406);
                    }

                    return;
                }

                byte[] dstBytes = new byte[dwDataLen];
                Marshal.Copy(pBuf, dstBytes, 0, dwDataLen);
                ReaderBuffMgr.DataBuilder(dstBytes, Environment.TickCount); //build the buffer on each invoke

                if ((ReaderBuffMgr.RawBytes[0] == 0x01) && (ReaderBuffMgr.RawBytes[1] == 0x02) && (ReaderBuffMgr.RawBytes[2] == 0x5D) && (ReaderBuffMgr.RawBytes[3] == 0x4C) && (ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng - 1] == 0x04))
                {
                    Lng = ReaderBuffMgr.PrevLng; // running total

                    if (Scanbuff.Length >= 1500)
                    {
                        Array.Clear(Scanbuff, 0, Scanbuff.Length);
                        MessageBox.Show("Clear performed!");
                    }

                    Array.Copy(ReaderBuffMgr.RawBytes, 0, Scanbuff, 0, Lng);
                    this.Invoke(new EventHandler(PDF417Process)); //parse the buffer.
                    //ReaderBuffMgr.PrevLng = 0;
                    //ReaderBuffMgr.count = 0;
                }
            }
            catch (Exception)
            {
            }
        }

        private void PDF417Process(object o, EventArgs e)
        {
            try
            {
                string version = TokenScanner.TkwkScan.GetRev(); // New function to test version match for TokenScanner and PDF417 dlls

                if (version.Length > 3) //the correct matching rev string will be returned of length=3, eg "3.4" or else a longer "mismatch..." errormessage
                {
                    MessageBox.Show(version);
                    return;
                }

                ParseRet = TokenScanner.TkwkScan.PDFParse(Scanbuff);

                if (ParseRet == 2048)
                {
                    Array.Clear(Scanbuff, 0, Scanbuff.Length);
                    return;
                }


                ClearImage();
                if (ParseRet == 99)
                {
                    Array.Clear(Scanbuff, 0, Scanbuff.Length);
                    MessageBox.Show("Scanner License Invalid");
                    return;
                }

                if (ParseRet == 98)
                {
                    Array.Clear(Scanbuff, 0, Scanbuff.Length);
                    MessageBox.Show("Input buffer larger than 1500 bytes");
                    return;
                }

                if (ParseRet == 96)
                {
                    Array.Clear(Scanbuff, 0, Scanbuff.Length);
                    MessageBox.Show("Fake ID by PartiTek");
                }

                AppContext.Instance().Content.IsImage = "0";

                txtTitle.Text = TkwkScan.Title;
                txtFirstName.Text = TkwkScan.FirstName;
                txtMidName.Text = TkwkScan.MidName;
                txtLastName.Text = TkwkScan.LastName;

                txtAddress.Text = TkwkScan.Address + " " + TkwkScan.Addr2;
                txtCity.Text = TkwkScan.City;
                txtState.Text = TkwkScan.State;
                txtZip.Text = TkwkScan.Zip;

                txtID.Text = TkwkScan.IDnum;
                txtEXP.Text = TkwkScan.EXP;
                txtDobMMDD.Text = TkwkScan.DobMMDD;
                txtDobYear.Text = TkwkScan.DobYear;
                txtDOB.Text = TkwkScan.DobMMDD + TkwkScan.DobYear;

                txtState.Text = TkwkScan.State;
                txtNationality.Text = TkwkScan.Race;
                txtIssueDate.Text = TkwkScan.DocIssueDate;

                txtHt.Text = TkwkScan.Height;
                txtEye.Text = TkwkScan.Eye;
                txtSex.Text = TkwkScan.Sex;
                txtClass.Text = TkwkScan.Cla;
                txtRestrict.Text = TkwkScan.Restr;
                txtEndors.Text = TkwkScan.Endrs;
                txtWt.Text = TkwkScan.Weight;
                txtHair.Text = TkwkScan.Hair;

                AppContext.Instance().Content.SetContent();
                AppContext.Instance().Content.CurrentImageContent = string.Empty;
                AppContext.Instance().Content.PreviousContent = string.Empty;
                timClearFields.Enabled = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show("PDF417Process : " + ex.Message);
            }
            finally
            {
                Array.Clear(Scanbuff, 0, Scanbuff.Length); //clear local buffer for next scan
            }
        }

        #endregion

        #region M420 Controls

        private void Init420()
        {
            // Initialize Scanner access to parsing
            OK = UpdateControls();
            if (OK)
            {
                RegisterCallBack(true); //Register the Data Receive Call back
            }

        }
        #endregion

        public static void SetCertPolicy()
        {
            ServicePointManager.ServerCertificateValidationCallback += RemoteCertValidate;
        }

        private static bool RemoteCertValidate(object sender, X509Certificate cert, X509Chain chain,
            SslPolicyErrors error)
        {
            // trust any cert!!!
            return true;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            AppContext.Instance().Content.SetContent();
            AppContext.Instance().Content.CurrentImageContent = string.Empty;
            AppContext.Instance().Content.PreviousContent = string.Empty;
        }
    }
}
