﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using System.IO;

namespace KPL
{
    public sealed class Utilities
    {
        private Utilities()
        {
        }

        public static Bitmap RotateImage(Bitmap image, float angle)
        {
            return Utilities.RotateImage(image, new PointF((float)image.Width / 2f, (float)image.Height / 2f), angle);
        }
        public static Bitmap RotateImage(Bitmap image, PointF offset, float angle)
        {
            if (image == null)
            {
                throw new ArgumentNullException("image");
            }
            Bitmap bitmap = new Bitmap(image.Width, image.Height, PixelFormat.Format16bppRgb565);
            bitmap.SetResolution(image.HorizontalResolution, image.VerticalResolution);
            Graphics graphics = Graphics.FromImage(bitmap);
            graphics.TranslateTransform(offset.X, offset.Y);
            graphics.RotateTransform(angle);
            graphics.TranslateTransform(-offset.X, -offset.Y);
            graphics.DrawImage(image, new PointF(0f, 0f));
            return bitmap;
        }
        public static Bitmap RotateImage(Bitmap image, PointF offset, byte bAngle)
        {
            if (image == null)
            {
                throw new ArgumentNullException("image");
            }
            float angle;
            if (bAngle > 127)
            {
                angle = (float)((double)(bAngle - 127) * 0.1);
            }
            else
            {
                if (bAngle == 127)
                {
                    angle = 0f;
                }
                else
                {
                    angle = -(float)((double)(127 - bAngle) * 0.1);
                }
            }
            Bitmap bitmap = new Bitmap(image.Width, image.Height, PixelFormat.Format16bppRgb565);
            bitmap.SetResolution(image.HorizontalResolution, image.VerticalResolution);
            Graphics graphics = Graphics.FromImage(bitmap);
            graphics.TranslateTransform(offset.X, offset.Y);
            graphics.RotateTransform(angle);
            graphics.TranslateTransform(-offset.X, -offset.Y);
            graphics.DrawImage(image, new PointF(0f, 0f));
            return bitmap;
        }
        public static byte ConverDEGtoByte(float angle)
        {
            byte result;
            if (angle > 0f)
            {
                if (Math.Round((double)(angle * 10f)) > 127.0)
                {
                    result = 255;
                }
                else
                {
                    result = (byte)(127.0 + Math.Round((double)(angle * 10f)));
                }
            }
            else
            {
                angle = Math.Abs(angle);
                if (Math.Round((double)(angle * 10f)) > 127.0)
                {
                    result = 0;
                }
                else
                {
                    result = (byte)(127.0 - Math.Round((double)(angle * 10f)));
                }
            }
            return result;
        }
    }
    public static class CSSNUtilities
    {
        public static int GetRegionIntValue(string region)
        {
            switch (region)
            {
                case "United States":
                    return 0;
                case "Australia":
                    return 4;
                case "Asia":
                    return 5;
                case "Canada":
                    return 1;
                case "America":
                    return 2;
                case "Europe":
                    return 3;
                case "Africa":
                    return 7;
                case "General doc.":
                    return 6;
                default:
                    return -1;
            }
        }

        public static void Alert(string message)
        {
            if (!string.IsNullOrEmpty(message))
            {
                MessageBox.Show(message, "Id Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public static bool IsCalibrateButtonRequired(int scanner_type)
        {
            bool ret = false;
            if (scanner_type == CSSNConstants.CSSN_600 || scanner_type == CSSNConstants.CSSN_800 || scanner_type == CSSNConstants.CSSN_800N || scanner_type == CSSNConstants.CSSN_1000 || scanner_type == CSSNConstants.CSSN_2000 || scanner_type == CSSNConstants.CSSN_2000N || scanner_type == CSSNConstants.CSSN_800E || scanner_type == CSSNConstants.CSSN_800EN || scanner_type == CSSNConstants.CSSN_3000 || scanner_type == CSSNConstants.CSSN_800DX || scanner_type == CSSNConstants.CSSN_800DXN || scanner_type == CSSNConstants.CSSN_IP || scanner_type == CSSNConstants.CSSN_1000N || scanner_type == CSSNConstants.CSSN_3000DN || scanner_type == CSSNConstants.CSSN_3100 || scanner_type == CSSNConstants.CSSN_3100N)
            {
                ret = true;
            }
            return ret;
        }

        public static bool IsPassportScanner(int scanner_type)
        {
            bool ret = false;
            if ((scanner_type == CSSNConstants.CSSN_1000 || scanner_type == CSSNConstants.CSSN_1000N || scanner_type == CSSNConstants.CSSN_PASS ||
               scanner_type == CSSNConstants.CSSN_PASS_R2))
            {
                ret = true;
            }
            return ret;
        }

        public static Image _GetImageFromBuffer(string imagebuffer)
        {
            byte[] imgBuffer;
            // If image buffer found process buffer
            if (imagebuffer != null)
            {
                // Change image buffer to byte[] to be  able to save in DataBase.
                char[] tempBuffer = imagebuffer.ToCharArray();
                byte[] imageDataBuffer = new byte[tempBuffer.Length];
                for (int i = 0; i < tempBuffer.Length; i++)
                {
                    imageDataBuffer[i] = (byte)tempBuffer[i];
                }
                imgBuffer = imageDataBuffer;
                MemoryStream ms = new MemoryStream(imgBuffer);
                Image returnImage = Image.FromStream(ms);
                ms.Close();
                return returnImage;
            }
            else
            {
                return null;
            }
        }

        public static Image _GetImageFromFile(string filePath)
        {
            if (!File.Exists(filePath))
                return null;
            if (filePath.Contains(".ejpg"))
            {
                return null;
            }
            FileStream fs = new FileStream(filePath, FileMode.Open);
            Image returnImage = Image.FromStream(fs);
            fs.Close();
            fs = null;
            return returnImage;
        }
    }

}
